@@ -1,3 +1,4 @@
+from math import log10
 from os import get_terminal_size
 from os import makedirs
 from os.path import join as path_join
@@ -24,6 +25,10 @@ from .database import update
 from .settings import setting_read
 
 
+class UnknownFolder(Exception):
+    pass
+
+
 def cookies_load(api: FAAPI, cookie_a: str, cookie_b: str):
     api.load_cookies([
         {"name": "a", "value": cookie_a},
@@ -62,6 +67,10 @@ def user_new(db: Connection, user: str):
     db.commit()
 
 
+def submission_check(db: Connection, sub_id: int) -> bool:
+    return bool(select(db, "SUBMISSIONS", ["ID"], "ID", sub_id).fetchone())
+
+
 def submission_save(db: Connection, sub: Sub, sub_file: Optional[bytes]):
     sub_ext: str = ""
 
@@ -100,7 +109,7 @@ def submission_save(db: Connection, sub: Sub, sub_file: Optional[bytes]):
 def submission_download_file(api: FAAPI, sub_file_url: str, speed: int = 100) -> Optional[bytes]:
     bar_length: int = 10
     bar_pos: int = 0
-    print("[" + (" " * bar_length) + "]", end=("\b" * bar_length) + "\b", flush=True)
+    print("[" + (" " * bar_length) + "]", end=("\b" * bar_length), flush=True)
 
     try:
         if not (file_stream := api.session.get(sub_file_url, stream=True)).ok:
@@ -140,6 +149,8 @@ def submission_download(api: FAAPI, db: Connection, sub_id: int) -> bool:
         raise
     except (Exception, BaseException):
         pass
+    finally:
+        print()
 
     if not sub.id:
         return False
@@ -156,7 +167,6 @@ def user_download(api: FAAPI, db: Connection, user: str, folder: str, stop: int
     page_n: int = 0
     user = user_clean_name(user)
     space_term: int = get_terminal_size()[0]
-    space_title: int = space_term - 14 - 17
     found_subs: int = 0
 
     downloader: Callable[[str, Union[str, int]], Tuple[List[SubPartial], Union[int, str]]] = lambda *x: ([], 0)
@@ -167,19 +177,22 @@ def user_download(api: FAAPI, db: Connection, user: str, folder: str, stop: int
     elif folder == "favorites":
         page = "next"
         downloader = api.favorites
+    else:
+        UnknownFolder(folder)
 
     user_new(db, user)
     user_add(db, user, "FOLDERS", folder.lower())
 
     while page:
         page_n += 1
-        print(f"{page_n:02d}    {user[:37]} ...", end="", flush=True)
+        space_title: int = space_term - 28 - (int(log10(page_n)) + 1)
+        print(f"{page_n}    {user[:space_title]} ...", end="", flush=True)
         user_subs, page = downloader(user, page)
         if not user_subs:
             print("\r" + (" " * 31), end="\r", flush=True)
         for i, sub in enumerate(user_subs, 1):
             print(
-                f"\r{page_n:02d}/{i:02d} {sub.id:010d} " +
+                f"\r{page_n}/{i:02d} {sub.id:010d} " +
                 f"{submission_clean_title(sub.title)[:space_title]:<{space_title}} ",
                 end="",
                 flush=True
@@ -191,9 +204,11 @@ def user_download(api: FAAPI, db: Connection, user: str, folder: str, stop: int
                 print(f"[{'FOUND':^10}]")
                 if stop and (found_subs := found_subs + 1) >= stop:
                     return subs_total, subs_failed
+            elif submission_check(db, sub.id):
+                print(f"[{'FOUND':^10}]")
+                user_add(db, user, folder.upper(), str(sub.id).zfill(10))
             elif submission_download(api, db, sub.id):
                 user_add(db, user, folder.upper(), str(sub.id).zfill(10))
                 subs_total += 1
-                print()
 
     return subs_total, subs_failed
